/**************************************************************************
    Copyright (C), 2002-2007, Taisuke Ozaki.
    The program package follows the terms of GNU GPL. 

  adpack.c:

     adpack.c (Atomic Density functional program PACKage) is a program
     package for atomic density functional calculations.

     This program includes
      1) all electron self-consistent calculations for an atom,
      2) generations of norm conserving pseudo potentials by using
         BHS (Bachelet, Hamann, and Schluter) or TM (Troullier and
         Martins) methods,
      3) calculations of charge density for the partial core correction,
      4) calculations of pseudo atomic orbitals for confinement pseudo
         potentials.  

  Log of adpack.c:

     10/Dec/2002  Released by T.Ozaki

**************************************************************************/

#include <stdio.h>
#include <math.h>
#include "adpack.h"
#include "Inputtools.h"

int main(int argc, char *argv[])
{
  static int i,k;



  /*
  { 

  static char Atom_Symbol[200][4];
  static char AN[100],ctmp,c,fname1[100];
  static double tmp,atmp[10],atmp0[10];
  double virial,Ekin,Ekin_x,Ekin_c,EHart,Eec,Exc,Ecorr;
 
  FILE *fp,*fp1,*fp2;

  strcpy(Atom_Symbol[  1], "H");
  strcpy(Atom_Symbol[  2], "He");
  strcpy(Atom_Symbol[  3], "Li");
  strcpy(Atom_Symbol[  4], "Be");
  strcpy(Atom_Symbol[  5], "B");
  strcpy(Atom_Symbol[  6], "C");
  strcpy(Atom_Symbol[  7], "N");
  strcpy(Atom_Symbol[  8], "O");
  strcpy(Atom_Symbol[  9], "F");
  strcpy(Atom_Symbol[ 10], "Ne");
  strcpy(Atom_Symbol[ 11], "Na");
  strcpy(Atom_Symbol[ 12], "Mg");
  strcpy(Atom_Symbol[ 13], "Al");
  strcpy(Atom_Symbol[ 14], "Si");
  strcpy(Atom_Symbol[ 15], "P");
  strcpy(Atom_Symbol[ 16], "S");
  strcpy(Atom_Symbol[ 17], "Cl");
  strcpy(Atom_Symbol[ 18], "Ar");
  strcpy(Atom_Symbol[ 19], "K");
  strcpy(Atom_Symbol[ 20], "Ca");
  strcpy(Atom_Symbol[ 21], "Sc");
  strcpy(Atom_Symbol[ 22], "Ti");
  strcpy(Atom_Symbol[ 23], "V");
  strcpy(Atom_Symbol[ 24], "Cr");
  strcpy(Atom_Symbol[ 25], "Mn");
  strcpy(Atom_Symbol[ 26], "Fe");
  strcpy(Atom_Symbol[ 27], "Co");
  strcpy(Atom_Symbol[ 28], "Ni");
  strcpy(Atom_Symbol[ 29], "Cu");
  strcpy(Atom_Symbol[ 30], "Zn");
  strcpy(Atom_Symbol[ 31], "Ga");
  strcpy(Atom_Symbol[ 32], "Ge");
  strcpy(Atom_Symbol[ 33], "As");
  strcpy(Atom_Symbol[ 34], "Se");
  strcpy(Atom_Symbol[ 35], "Br");
  strcpy(Atom_Symbol[ 36], "Kr");
  strcpy(Atom_Symbol[ 37], "Rb");
  strcpy(Atom_Symbol[ 38], "Sr");
  strcpy(Atom_Symbol[ 39], "Y");
  strcpy(Atom_Symbol[ 40], "Zr");
  strcpy(Atom_Symbol[ 41], "Nb");
  strcpy(Atom_Symbol[ 42], "Mo");
  strcpy(Atom_Symbol[ 43], "Tc");
  strcpy(Atom_Symbol[ 44], "Ru");
  strcpy(Atom_Symbol[ 45], "Rh");
  strcpy(Atom_Symbol[ 46], "Pd");
  strcpy(Atom_Symbol[ 47], "Ag");
  strcpy(Atom_Symbol[ 48], "Cd");
  strcpy(Atom_Symbol[ 49], "In");
  strcpy(Atom_Symbol[ 50], "Sn");
  strcpy(Atom_Symbol[ 51], "Sb");
  strcpy(Atom_Symbol[ 52], "Te");
  strcpy(Atom_Symbol[ 53], "I");
  strcpy(Atom_Symbol[ 54], "Xe");
  strcpy(Atom_Symbol[ 55], "Cs");
  strcpy(Atom_Symbol[ 56], "Ba");
  strcpy(Atom_Symbol[ 57], "La");
  strcpy(Atom_Symbol[ 58], "Ce");
  strcpy(Atom_Symbol[ 59], "Pr");
  strcpy(Atom_Symbol[ 60], "Nd");
  strcpy(Atom_Symbol[ 61], "Pm");
  strcpy(Atom_Symbol[ 62], "Sm");
  strcpy(Atom_Symbol[ 63], "Eu");
  strcpy(Atom_Symbol[ 64], "Gd");
  strcpy(Atom_Symbol[ 65], "Tb");
  strcpy(Atom_Symbol[ 66], "Dy");
  strcpy(Atom_Symbol[ 67], "Ho");
  strcpy(Atom_Symbol[ 68], "Er");
  strcpy(Atom_Symbol[ 69], "Tm");
  strcpy(Atom_Symbol[ 70], "Yb");
  strcpy(Atom_Symbol[ 71], "Lu");
  strcpy(Atom_Symbol[ 72], "Hf");
  strcpy(Atom_Symbol[ 73], "Ta");
  strcpy(Atom_Symbol[ 74], "W");
  strcpy(Atom_Symbol[ 75], "Re");
  strcpy(Atom_Symbol[ 76], "Os");
  strcpy(Atom_Symbol[ 77], "Ir");
  strcpy(Atom_Symbol[ 78], "Pt");
  strcpy(Atom_Symbol[ 79], "Au");
  strcpy(Atom_Symbol[ 80], "Hg");
  strcpy(Atom_Symbol[ 81], "Tl");
  strcpy(Atom_Symbol[ 82], "Pb");
  strcpy(Atom_Symbol[ 83], "Bi");
  strcpy(Atom_Symbol[ 84], "Po");
  strcpy(Atom_Symbol[ 85], "At");
  strcpy(Atom_Symbol[ 86], "Rn");
  strcpy(Atom_Symbol[ 87], "Fr");
  strcpy(Atom_Symbol[ 88], "Ra");
  strcpy(Atom_Symbol[ 89], "Ac");
  strcpy(Atom_Symbol[ 90], "Th");
  strcpy(Atom_Symbol[ 91], "Pa");
  strcpy(Atom_Symbol[ 92], "U");
  strcpy(Atom_Symbol[ 93], "Np");
  strcpy(Atom_Symbol[ 94], "Pu");
  strcpy(Atom_Symbol[ 95], "Am");
  strcpy(Atom_Symbol[ 96], "Cm");
  strcpy(Atom_Symbol[ 97], "Bk");
  strcpy(Atom_Symbol[ 98], "Cf");
  strcpy(Atom_Symbol[ 99], "Es");
  strcpy(Atom_Symbol[100], "Fm");
  strcpy(Atom_Symbol[101], "Md");
  strcpy(Atom_Symbol[102], "No");
  strcpy(Atom_Symbol[103], "Lr");

  for (i=1; i<=103; i++){
    sprintf(AN,"%s0.alog",Atom_Symbol[i]);   

    sprintf(fname1,"tempf");
    fp1 = fopen(fname1,"w");
    fseek(fp1,0,SEEK_END);

    fp2 = fopen(AN,"r");
    if (fp2!=NULL){
      for (c=getc(fp2); c!=EOF; c=getc(fp2)){
        if (c!='=')  putc(c,fp1); 
      }
      fclose(fp2); 
    }
    fclose(fp1); 

    input_open(fname1);

    input_double("Ekin",&Ekin,0.0);
    input_double("EHart",&EHart,0.0);
    input_double("Eec",&Eec,0.0);
    input_double("Exc",&Exc,0.0);
    input_double("Ekin_x",&Ekin_x,0.0);
    input_double("Ekin_c",&Ekin_c,0.0);
    input_double("Ecorr",&Ecorr,0.0);

    virial = 2.0*(Ekin+Ekin_x+Ekin_c) + (EHart+Eec+Exc-Ekin_x-Ekin_c);

    printf("%4d %18.15f %18.15f %18.15f %18.15f\n",i,virial,Ecorr,Ekin,Ekin_c);

    input_close();
  }
  }
  exit(0);
  */









  /****************************************************
               Set of input parameters                
  ****************************************************/

  readfile(argv);

  /****************************************************
                     Initialize
  ****************************************************/

  Set_Init();

  /****************************************************
                All electron calculation
  ****************************************************/

  /* calculation of frozen core */
  if (Calc_Type==3){
    All_Electron(0);
    All_Electron(1);
  }

  /* all electron calculation by FEM */
  else if (Calc_Type==4){
    /* FEM_All_Electron(); */
  }
  else{
    i = Restart_load(0);
    if ( i==0 ) All_Electron(0);
    if ( (i==0 && strlen(restartfile)) )  Restart_save(0);
  }

  /****************************************************
                   Pseudo potentials
  ****************************************************/

  if ( (Calc_Type==1 || Calc_Type==2) && AtomNum!=0){

    Init_VPS();

    if      (VPP_switch==0)  BHS(0);
    else if (VPP_switch==1)  TM(0);
    else if (VPP_switch==2)  MR(0);

    if (VPP_switch!=4) Generate_VNL();

    /* Find_LESP(); */

    if (Calc_Type==1 && LogD_switch==1) Log_DeriF();
    if (Calc_Type==1 && ghost_check==1) ghost(0);
  }
  else if (AtomNum==0){
    Empty_VPS();
  }

  /****************************************************
                   Pseudo atomic orbitals
  ****************************************************/

  if (Calc_Type==2) Multiple_PAO(0);

  /****************************************************
                        Output 
  ****************************************************/

  Output();

  /****************************************************
                     a final message
  ****************************************************/

  printf("\nThe calculation was completed normally.\n");

}



